
#include <windows.h>  
#include "drawex.h"  
#include "commctrl.h"


#if defined (WIN32)
	#define IS_WIN32 TRUE
#else
	#define IS_WIN32 FALSE
#endif

#define IS_NT      IS_WIN32 && (BOOL)(GetVersion() < 0x80000000)
#define IS_WIN32S  IS_WIN32 && (BOOL)(!(IS_NT) && (LOBYTE(LOWORD(GetVersion()))<4))
#define IS_WIN95   (BOOL)(!(IS_NT) && !(IS_WIN32S)) && IS_WIN32

HINSTANCE hInst;   // current instance

LPCTSTR lpszAppName = "MyApp";
LPCTSTR lpszTitle   = "ImageList_DrawEx()"; 


BOOL RegisterWin95( CONST WNDCLASS* lpwc );


int APIENTRY WinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPTSTR lpCmdLine, int nCmdShow)
{
   MSG      msg;
   HWND     hWnd; 
   WNDCLASS wc;

   wc.style         = CS_HREDRAW | CS_VREDRAW;
   wc.lpfnWndProc   = (WNDPROC)WndProc;       
   wc.cbClsExtra    = 0;                      
   wc.cbWndExtra    = 0;                      
   wc.hInstance     = hInstance;              
   wc.hIcon         = LoadIcon (hInstance, lpszAppName); 
   wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
   wc.lpszMenuName  = lpszAppName;              
   wc.lpszClassName = lpszAppName;              

   if ( IS_WIN95 )
   {
      if ( !RegisterWin95( &wc ) )
         return( FALSE );
   }
   else if ( !RegisterClass( &wc ) )
      return( FALSE );

   hInst = hInstance; 

   hWnd = CreateWindow( lpszAppName, 
                        lpszTitle,    
                        WS_OVERLAPPEDWINDOW, 
                        CW_USEDEFAULT, 0, 
                        CW_USEDEFAULT, 0,  
                        NULL,              
                        NULL,              
                        hInstance,         
                        NULL               
                      );

   if ( !hWnd ) 
      return( FALSE );

   ShowWindow( hWnd, nCmdShow ); 
   UpdateWindow( hWnd );         

   while( GetMessage( &msg, NULL, 0, 0) )   
   {
      TranslateMessage( &msg ); 
      DispatchMessage( &msg );  
   }

   return( msg.wParam ); 
}


BOOL RegisterWin95( CONST WNDCLASS* lpwc )
{
   WNDCLASSEX wcex;

   wcex.style         = lpwc->style;
   wcex.lpfnWndProc   = lpwc->lpfnWndProc;
   wcex.cbClsExtra    = lpwc->cbClsExtra;
   wcex.cbWndExtra    = lpwc->cbWndExtra;
   wcex.hInstance     = lpwc->hInstance;
   wcex.hIcon         = lpwc->hIcon;
   wcex.hCursor       = lpwc->hCursor;
   wcex.hbrBackground = lpwc->hbrBackground;
   wcex.lpszMenuName  = lpwc->lpszMenuName;
   wcex.lpszClassName = lpwc->lpszClassName;

   // Added elements for Windows 95.
   //...............................
   wcex.cbSize = sizeof(WNDCLASSEX);
   wcex.hIconSm = LoadImage(wcex.hInstance, lpwc->lpszClassName, 
                            IMAGE_ICON, 16, 16,
                            LR_DEFAULTCOLOR );
			
   return RegisterClassEx( &wcex );
}

HIMAGELIST hList = NULL;

LRESULT CALLBACK WndProc( HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
static int nSel  = -1;

   switch( uMsg )
   {
      case WM_CREATE :
              InitCommonControls();

              // Create the image list.
              //.......................
              hList = ImageList_Create( 32, 32, ILC_COLOR | ILC_MASK, 1, 1 );
              if ( hList )
              {
                 HBITMAP hBmp;
                 
                 // Add an image and build a mask.
                 //...............................
                 hBmp = LoadBitmap( hInst, "BMP2" );
                 ImageList_AddMasked( hList, hBmp, RGB( 0, 255, 0 ) );
                 DeleteObject( hBmp );
              }
              break;

      case WM_PAINT :
              {
                 PAINTSTRUCT ps;

                 // Paint the two images on the client area.
                 //.........................................
                 BeginPaint( hWnd, &ps );
                 ImageList_DrawEx( hList, 0, ps.hdc, 10, 10, 0, 0, 
                                   CLR_DEFAULT, RGB( 255, 0, 0 ),
                                   ILD_NORMAL | (nSel == 0 ? ILD_BLEND25 : 0) );
                 EndPaint( hWnd, &ps );
              }
              break;

      case WM_COMMAND :
              switch( LOWORD( wParam ) )
              {
                 case IDM_SELECT :
                        nSel = (nSel == -1 ? 0 : -1);
                        InvalidateRect( hWnd, NULL, TRUE );
                        break;

                 case IDM_SETBKGRND :
                        DialogBox( hInst, "RGBDialog", hWnd, (DLGPROC)RGBDialog ); 
                        InvalidateRect( hWnd, NULL, TRUE );
                        break;

                 case IDM_ABOUT :
                        DialogBox( hInst, "AboutBox", hWnd, (DLGPROC)About );
                        break;

                 case IDM_EXIT :
                        DestroyWindow( hWnd );
                        break;
              }
              break;
      
      case WM_DESTROY :
              if ( hList )
                 ImageList_Destroy( hList );

              PostQuitMessage(0);
              break;

      default :
            return( DefWindowProc( hWnd, uMsg, wParam, lParam ) );
   }

   return( 0L );
}


LRESULT CALLBACK RGBDialog( HWND hDlg,           
                            UINT message,        
                            WPARAM wParam,       
                            LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               {
                  COLORREF color = ImageList_GetBkColor( hList );

                  SetDlgItemInt( hDlg, IDC_RED,   GetRValue( color ), FALSE );
                  SetDlgItemInt( hDlg, IDC_GREEN, GetGValue( color ), FALSE );
                  SetDlgItemInt( hDlg, IDC_BLUE,  GetBValue( color ), FALSE );
               }    
               return (TRUE);

       case WM_COMMAND:                              
               if ( LOWORD(wParam) == IDOK )
               {
                  ImageList_SetBkColor( hList, RGB( 
                                               GetDlgItemInt( hDlg, IDC_RED, NULL, FALSE ),
                                               GetDlgItemInt( hDlg, IDC_GREEN, NULL, FALSE ),
                                               GetDlgItemInt( hDlg, IDC_BLUE, NULL, FALSE ) ) );

                  EndDialog( hDlg, IDOK );        
                  return (TRUE);
               }
               else if ( LOWORD(wParam) == IDCANCEL)    
               {
                  EndDialog( hDlg, IDCANCEL );        
                  return (TRUE);
               }
               break;
   }

   return (FALSE); 
}


LRESULT CALLBACK About( HWND hDlg,           
                        UINT message,        
                        WPARAM wParam,       
                        LPARAM lParam)
{
   switch (message) 
   {
       case WM_INITDIALOG: 
               return (TRUE);

       case WM_COMMAND:                              
               if (   LOWORD(wParam) == IDOK         
                   || LOWORD(wParam) == IDCANCEL)    
               {
                       EndDialog(hDlg, TRUE);        
                       return (TRUE);
               }
               break;
   }

   return (FALSE); 
}
